/***********************************************************
cv.c - Construction des fichiers d'update pour Max.

Codes generaux
--------------
B nnn n		ecrit les initialisations;
		ex: B 100 0

b nnn n		entete de definition d'action.
		commencer chaque nouvelle definition d'action
		par ce code; il s'applique si les temps incrementaux ont
		change;
		Le programme 'genupd' produit une liste d'entetes; on donne
		l'etiquette de debut, de fin, et l'increment.

x		termine la partition.

Harmonizers
-----------
t n  Un|Dn		==	hgain[n] T	 hfreq[n] Un|Dn
				hdel[n] <MSdelay>
	- transposition U (Up) ou D (Down) de n 1/2 tons ou 1/4 de tons
	- MSdelay: delai en millisecondes, 
		lu dans le champ 'n' de l'ordre 'b nnn n'.

g n			==	hgain[n] F
	- couper le gain de l'harmonizer N

Frequency Shifters
------------------
les frequences sont donnees en hauteur MIDI (PitchMIDI); pour les modulations
d'amplitudes, les tres basses frequences sont donnees en Hertz (frequences 
inferieures au pitchMIDI 21.

+ F		== 	ctof F 	R2_fpos F	R2_rtof F	R2_ftor F
- F		== 	ctof F 	R2_fneg F	R2_rtof F	R2_ftor F
+ T <PitchMIDI>	== 	ctof F	R2_fpos T 	R2_fshift <Hexa>
- T <PitchMIDI>	== 	ctof F 	R2_fneg T 	R2_fshift <Hexa>

Reverberations
--------------
R n		==	revgate n
 
r REVI		==	revgn REVI	 revlp REVLPI
r REVD 		==	revgn REVD	 revlp REVLPD

Synthese additive
-----------------
f n h|m		==	fre[n] <hexa>
		h =	frequence en Hz
		m =	pitch MIDI

d n p|a		==	v_depth[n] <hexa>
		p =	pourcentage de la frequence centrale
		a =	valeur absolue en Hz

v n <freqHz>	==	v_vit[n] <hexa>

w n {1,16}	==	selection d'une wavetable #{1,16}
			wt[0,15] <hexa>

Codes de liaison entre modules (XtoY) (toujours suivis de T ou F)
------------------------------------
	.cf T|F	==	R2_ctof T|F
	.df	==	R2_dtof
	.dh	==	R1_dtoh
	.dr	==	R2_dtor
	.fr	==	R2_ftor
	.hr	==	R2_htor
	.hf	==	htof
	.rf	==	R2_rtof
	.rh	==	R1_rtoh
	.sr	==	stor
	.sh	==	stoh
	.sf	==	stof

syntaxe 
-------
$cv	 UpdateFile(fichier entree)	 UpdateFile.a(fichier sortie)

Codes utilises pour les Update lists
------------------------------------
('*' utilises a l'interieur d'un autre code)
a*, b, B, c, d, D, e, E*, f, F*, g, h, i, m, p*, r, R, t, T*, v, w
+*, -*, ., .fr, .rf

*****************************************************************/

#include <math.h>
#include <stdio.h>
#include <strings.h>
#define CONST 0x1000000
#define FOREVER for(;;)
char strans[10];
char sdel[10];
float atof(), getnum(); 
FILE *fi;
int timenow, beatnow;

main(argc,argv)
int argc;
char *argv[];
{
	float freq, fs, percnt, depth, intval, xf;
	int f,ilun,iosc;
	int i,n,n1,n2,icpt=0;
	int c, cminus, nwt;
	float CalIntval();
	float CalPercnt();
	float FMidi[110];
	char car;
	char nstr[10];			/* input string */
	char anyname[20];
	char ssign[10];
	char TorF[10];
	char TorF2[10];
	char rffr[10];

	FILE *fo;
/*	FILE *fo2;	*/

/* check command string : PROG filin filout  (3 arguments) */
	if (argc != 3){
		printf("Usage: PROG filin filout \n");
		exit(0);
		}
/* test sur filin: existe? si oui, ouvrir, sinon exit */
	if ((fi=fopen (argv[1],"r"))==NULL){
		printf("ERR: le fichier %s n'existe pas\n",argv[1]);
		exit(0);
		}
/* ouvrir fichier de sortie */
	fo=fopen(argv[2],"w");

/* ouvrir fichier de test */
	/*fo2=fopen("test","w");*/


/***********************************************************************/


FMidi[21]= 27.501543; FMidi[22]= 29.136869; FMidi[23]= 30.869436;
FMidi[24]= 32.705025; FMidi[25]= 34.649765; FMidi[26]= 36.710144;
FMidi[27]= 38.893040; FMidi[28]= 41.205738; FMidi[29]= 43.655956;
FMidi[30]= 46.251873; FMidi[31]= 49.002151; FMidi[32]= 51.915970;
FMidi[33]= 55.003052; FMidi[34]= 58.273701; FMidi[35]= 61.738834;
FMidi[36]= 65.410011; FMidi[37]= 69.299492; FMidi[38]= 73.420250;
FMidi[39]= 77.786041; FMidi[40]= 82.411438; FMidi[41]= 87.311874;
FMidi[42]= 92.503708; FMidi[43]= 98.004257; FMidi[44]= 103.831886;
FMidi[45]= 110.006050; FMidi[46]= 116.547348; FMidi[47]= 123.477608;
FMidi[48]= 130.819962; FMidi[49]= 138.598907; FMidi[50]= 146.840424;
FMidi[51]= 155.572006; FMidi[52]= 164.822784; FMidi[53]= 174.623642;
FMidi[54]= 185.007294; FMidi[55]= 196.008392; FMidi[56]= 207.663651;
FMidi[57]= 220.011963; FMidi[58]= 233.094543; FMidi[59]= 246.955063;
FMidi[60]= 261.639771; FMidi[61]= 277.197662; FMidi[62]= 293.680664;
FMidi[63]= 311.143799; FMidi[64]= 329.645355; FMidi[65]= 349.247070;
FMidi[66]= 370.014374; FMidi[67]= 392.016571; FMidi[68]= 415.327087;
FMidi[69]= 440.023712; FMidi[70]= 466.188873; FMidi[71]= 493.909882;
FMidi[72]= 523.279236; FMidi[73]= 554.395020; FMidi[74]= 587.361023;
FMidi[75]= 622.287292; FMidi[76]= 659.290405; FMidi[77]= 698.493835;
FMidi[78]= 740.028381; FMidi[79]= 784.032715; FMidi[80]= 830.653687;
FMidi[81]= 880.046875; FMidi[82]= 932.377136; FMidi[83]= 987.819153;
FMidi[84]= 1046.557861; FMidi[85]= 1108.789429; FMidi[86]= 1174.721436;
FMidi[87]= 1244.573975; FMidi[88]= 1318.580078; FMidi[89]= 1396.986816;
FMidi[90]= 1480.055908; FMidi[91]= 1568.064575; FMidi[92]= 1661.306519;
FMidi[93]= 1760.092896; FMidi[94]= 1864.753418; FMidi[95]= 1975.637329;
FMidi[96]= 2093.114746; FMidi[97]= 2217.577637; FMidi[98]= 2349.441650;
FMidi[99]= 2489.146484; FMidi[100]= 2637.158691; FMidi[101]= 2793.972168;
FMidi[102]= 2960.110352; FMidi[103]= 3136.127441; FMidi[104]= 3322.611084;
FMidi[105]= 3520.183594; FMidi[106]= 3729.504395; FMidi[107]= 3951.066;
FMidi[108]= 4186.009;


FOREVER 
{

fscanf(fi,"%s",nstr);

if (nstr[0] == 'B')
	{
	fscanf(fi,"%d %d",&n1,&n2); /*read and flush n2(compatibility)*/

fprintf(fo,"%d %d000  ",n1,n1);
/* fprintf(fo,"R1_hfreq[0] 0 R1_hfreq[1] 0 R1_hfreq[2] 0 R1_hfreq[3] 0 ");
fprintf(fo,"R1_hdel[0] 0 R1_hdel[1] 0 R1_hdel[2] 0 R1_hdel[3] 0 ");
fprintf(fo,"R1_hgain[0] 0 R1_hgain[1] 0 R1_hgain[2] 0 R1_hgain[3] 0 ");
fprintf(fo,"R1_hgtc 2000 R1_rtoh F R1_dtoh T ");
fprintf(fo,"R1_etof F R2_fshift F R2_clavshift F R2_etof F ");
fprintf(fo,"R2_ctof F R2_dtof F R2_rtof T R2_htor T ");
fprintf(fo,"R2_stof F R2_stor T R1_stoh T ");
fprintf(fo,"R2_ftor F R2_dtor T R2_revgn REVD R2_revlp REVLPD ");
fprintf(fo,"stof F stoh F stor F ");
fprintf(fo,"R2_htof T "); */

	}

else if (nstr[0] == 'b')
	{
	freq = -1.; /* RAZ */
	depth = -1.;
	icpt++;
	fscanf(fi,"%d %d",&n1,&n2);
	beatnow = n1;
	timenow = 1000 * n1 + n2;
	if (n2 == 0)
		fprintf(fo,"\n%d %d000 ",n1,n1);
	else
	if (n2 < 10)
		fprintf(fo,"\n%d %d00%d ",n1,n1,n2);
	else
	if (n2 < 100)
		fprintf(fo,"\n%d %d0%d ",n1,n1,n2);
	else
	if (n2 < 1000)
		fprintf(fo,"\n%d %d%d ",n1,n1,n2);
	else
	if (n2 < 10000)
		fprintf(fo,"\n%d %d%d ",n1,n1/10,n2);
	}

/************* pour harmonizers, delays et reverb ***************/

else if (nstr[0] == 't')	/* hfreq[n] Transposition */
	{
	fscanf(fi,"%d",&n);	/* entrer osc. courant */
	fscanf(fi,"%s",strans);	/* transposition string (U/D) */

	if (n2 != 0)
fprintf(fo," hgain[%d] T hfreq[%d] %s  hdel[%d] MS%d ",n,n,strans,n,n2);
	else 
fprintf(fo," hgain[%d] T hfreq[%d] %s  hdel[%d] 0 ",n,n,strans,n);

	}

else if (nstr[0] == 'g')	/* couper harmonizer: hgain[n] F */
	{
	fscanf(fi,"%d",&n);	/* entrer osc. courant */
	fprintf(fo," hgain[%d] F ",n);
	}

else if (nstr[0] == 'R')		/* R n --> revgate n */
	{
	fscanf(fi,"%d",&n);	/* entrer val. amp. rev. */
	fprintf(fo," revgate %d ",n);
	}

else if (nstr[0] == 'r')	/* r REVI --> revgn REVI revlp REVLPI */
				/* r REVD --> revgn REVD revlp REVLPD */
	{			
	fscanf(fi,"%s",strans);	/* code REVI/D */
	if (strans[0] == 'I')
		fprintf(fo," revgn REVI revlp REVLPI ");
     else
	if (strans[0] == 'D')
	   {			
	   	fscanf(fi,"%d",&n);
		fprintf(fo," revgn REVD%d revlp REVLPD ",n);
	   }
	}

/************* pour frequency shifters   ***********************/
/*
	+|- F	
	+|- T <pitchMIDI>
	.rf T|F
	.fr T|F
*/
	
else if (nstr[0] == '+')		/* ctof T/F */
	{
	fprintf(fo," ctof F ");

	fscanf(fi,"%s",TorF);
	if (TorF[0] == 'F')
		fprintf(fo,"R2_fpos F R2_rtof F R2_ftor F ");

	else if (TorF[0] == 'T')
		{
	fscanf(fi,"%d",&i);
	if (i > 20)
		fs=FMidi[i];
	else fs=(float)i;
	fprintf(fo,"R2_fpos T R2_fshift %x ",(int)((CONST / 32000.)*fs));
		}
	}

else if (nstr[0] == '-')
	{
	fprintf(fo," ctof F ");

	fscanf(fi,"%s",TorF);
	if (TorF[0] == 'F')
		fprintf(fo,"R2_fneg F R2_rtof F R2_ftor F ");

	else if (TorF[0] == 'T')
		{
	fscanf(fi,"%d",&i);
	if (i > 20)
		fs=FMidi[i];
	else fs=(float)i;
	fprintf(fo,"R2_fneg T R2_fshift %x ",(int)((CONST / 32000.)*fs));
		}
	}

/****	Codes precedes par un '.'	****/
/* 	code .rf T|F ==	R2_rtof T|F	*/
else if (nstr[0] == '.' && nstr[1] == 'r' && nstr[2] == 'f')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R2_rtof %s ",TorF);
	}

/* 	code .fr T|F == R2_ftor T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'f' && nstr[2] == 'r')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R2_ftor %s ",TorF);
	}

/* 	code .hr T|F == R2_htor T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'h' && nstr[2] == 'r')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R2_htor %s ",TorF);
	}

/* 	code .dr T|F ==	R2_dtor T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'd' && nstr[2] == 'r')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R2_dtor %s ",TorF);
	}

/* 	code .df T|F == R2_dtof T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'd' && nstr[2] == 'f')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R2_dtof %s ",TorF);
	}

/* 	code .sr T|F == stor T|F 15/1/87	*/
else if (nstr[0] == '.' && nstr[1] == 's' && nstr[2] == 'r')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," stor %s ",TorF);
	}

/* 	code .sh T|F == stoh T|F 15/1/87	*/
else if (nstr[0] == '.' && nstr[1] == 's' && nstr[2] == 'h')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," stoh %s ",TorF);
	}

/* 	code .sf T|F == stof T|F 15/1/87 	*/
else if (nstr[0] == '.' && nstr[1] == 's' && nstr[2] == 'f')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," stof %s ",TorF);
	}


/* 	code .fh T|F == ftoh T|F 15/1/87 	*/
else if (nstr[0] == '.' && nstr[1] == 's' && nstr[2] == 'f')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," ftoh %s ",TorF);
	}


/* 	code .rh T|F == R1_rtoh T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'r' && nstr[2] == 'h')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R1_rtoh %s ",TorF);
	}

/* 	code .dh T|F == R1_dtoh T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'd' && nstr[2] == 'h')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R1_dtoh %s ",TorF);
	}

/* 	code .hf T|F == R2_htof T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'h' && nstr[2] == 'f')
	{ 
	fscanf(fi,"%s",TorF);
	fprintf(fo,"R2_htof %s ",TorF);
	}

/* 	code .cf T|F == R2_ctof T|F 	*/
else if (nstr[0] == '.' && nstr[1] == 'c' && nstr[2] == 'f')
	{
	fscanf(fi,"%s",TorF);
	fprintf(fo," R2_ctof %s ",TorF);
	}


/************* pour synthese additive - PLAQUE 7 ***********/
/* fre[n] */
else if (nstr[0] == 'f')		
	{
	fscanf(fi,"%d",&iosc);
	fprintf(fo," fre2[%d] ",iosc);
	}

else if (nstr[0] == 'h')		/* freq. en Hz */
	{
	fscanf(fi,"%f",&xf);
	freq=xf; /* keep freq value for vin */
	fprintf(fo,"%x ",(int)((CONST/32000.)*xf));
	}

else if (nstr[0] == 'm')		/* freq. MIDI */
	{
	fscanf(fi,"%d",&i);
	xf=FMidi[i];
	freq=xf; /* keep freq value for vib */
	fprintf(fo,"%x ",(int)((CONST/32000.)*xf));
	}

else if (nstr[0] == 'd')		/* depth vib */
	{
	fscanf(fi,"%d",&ilun);
	fprintf(fo," v_depth[%d] ",ilun);

	fscanf(fi,"%s",nstr); /* read next char */
	if (nstr[0] == 'p') /* profondeur = pourcentage de freq. centrale */
		{
		fscanf(fi,"%f",&xf);
		depth=CalPercnt(freq,xf); /* conversion en Hz */
		}
	else if (nstr[0] == 'a') /* profondeur = valeur Absolue en freq. */
		{
		fscanf(fi,"%f",&xf);
		depth=CalIntval(freq,xf); /* conversion en Hz */
		}
	/* ecrire pour verif */
/*	fprintf(fo2,"note %d, freq= %f, depth= %f ",icpt,freq,depth);*/
	fprintf(fo,"%x",(int)((CONST/32000.)*depth));
	}

/* selection d'une wt. de 1 a 16 */
else if (nstr[0] == 'w')
	{
	fscanf(fi,"%d",&ilun);
	fscanf(fi,"%d",&nwt);
	if (nwt < 0 || nwt > 15)
	  printf(" ERR: wt[%d] invalide, etiquette %d\n",ilun,n1);
	else
	{
	  if (ilun == 20) fprintf(fo," cha_tab ");
	  else fprintf(fo," wt[%d] ",ilun);
	  switch(nwt)
	   {
	   case 1: fprintf(fo," 101010 ");break;
	   case 2: fprintf(fo," 111111 ");break;
	   case 3: fprintf(fo," 121212 ");break;
	   case 4: fprintf(fo," 131313 ");break;
	   case 5: fprintf(fo," 141414 ");break;
	   case 6: fprintf(fo," 151515 ");break;
	   case 7: fprintf(fo," 161616 ");break;
	   case 8: fprintf(fo," 171717 ");break;
	   case 9: fprintf(fo," 181818 ");break;
	   case 10: fprintf(fo," 191919 ");break;
	   case 11: fprintf(fo," 1a1a1a ");break;
	   case 12: fprintf(fo," 1b1b1b ");break;
	   case 13: fprintf(fo," 1c1c1c ");break;
	   case 14: fprintf(fo," 1d1d1d ");break;
	   case 15: fprintf(fo," 1e1e1e ");break;
	   case 16: fprintf(fo," 1f1f1f ");break;
	   default:
	      printf(" ERR: wavetable %d invalide, etiquette %d\n",nwt,n1);
	      break;
	   }
	  }
	}


else if (nstr[0] == 'v')		/* vit vib */
	{
	fscanf(fi,"%d",&ilun);
	fprintf(fo," v_vit[%d] ",ilun);
	fscanf(fi,"%f",&xf);
	fprintf(fo,"%x ",(int)((CONST/32000.)*xf));
	}

else if (nstr[0] == 'i')		/* inc luni */
	{
	fscanf(fi,"%s",nstr); /* read next char */
		if (nstr[0] == 'P')
		{
			fscanf(fi,"%d",&ilun);
			fprintf(fo,"  inc[%d] PI ",ilun);
		}
		else if (nstr[0] == 'N')
		{
			fscanf(fi,"%d",&ilun);
			fprintf(fo,"  inc[%d] NI ",ilun);
		}
	}
	
else if (nstr[0] == 'e')		/* fin luni */
	{
	fscanf(fi,"%s",nstr); /* read next char */
	if (nstr[0] == 'N')
		{ 
		fscanf(fi,"%d",&ilun);
		fprintf(fo," fin[%d] NE ",ilun);
		} 
		else
		{
		fscanf(fi,"%d",&ilun);
			fprintf(fo," fin[%d] PE%s ",ilun,nstr);
		}
	}

else if (nstr[0] == 'E')
	{
		float temps, number = getnum();
		float lastfin = getnum();
		int totalmsec = 0, didone = 0;
		while ((temps = getnum()) > 0)
		{
			float fin = getnum(), finc= 
			     0x100*(fin - lastfin)/(temps>.01?temps-.005:.005);
			int inc = 0xffffff & (int) finc;
			if (inc == 0)
			{
				if (finc < 0) inc = -1;
				else if (finc > 0) inc = 1;
			}
			if (totalmsec) 
			{
				fprintf(fo, "\n%d %d ", beatnow, 
							timenow + totalmsec);
				didone = 1;
			}
			totalmsec = totalmsec + 1000 * temps;
			fprintf(fo, "inc[%d] %x fin[%d] %x ",
				(int)number,
				inc,
				(int)number,
				(int)(0x7fffff * fin));
			lastfin = fin;
		}
		if (didone) fprintf(fo, "\n%d %d ", beatnow, timenow);
	}


else if (nstr[0] == 'H')		/* synth to harms */
	{
	int x;
	fscanf(fi, "%d", &x);
	fprintf(fo, "STOHARMS%d ", x);
	}
else if (nstr[0] == 'L')		/* literal */
	{
	char str[20];
	fscanf(fi, "%s", str);
	fprintf(fo, "%s ", str);
	}
else if (nstr[0] == 'U')		/* update */
	{
	char str[20];
	fscanf(fi, "%s", str); fprintf(fo, "%s ", str);
	fscanf(fi, "%s", str); fprintf(fo, "%s ", str);
	}
else if (nstr[0] == 'C')		/* chapo frequencies */
	{
	int a;
	float b;
	fscanf(fi, "%d %f", &a,&b);
	fprintf(fo,"fre1[%d] %x ",a, (int)((CONST / 32000.)*b));
	}
else if (nstr[0] == 'x')		/* fin */
	{
	fprintf(fo, "\n");
	break;
	}
else fprintf(stderr, "unknown command: %s\n", nstr);
/* else
	{
	fscanf(fi,"%d",&f);
	fprintf(fo," %d ",f);
	} */

} /* end of forever */
fclose (fi);
fclose (fo);
} /* end of main */

/* fonction pour convertir intervalle en Hz en profondeur de vibrato */
float CalIntval(ffreq,fintval)
float ffreq,fintval;
{
float x1;
	x1=ffreq*pow(2.0,1./(6.*fintval));
	x1=x1-ffreq;
	return(x1);
}

/* conversion du pourcentage de la freq. centrale en profondeur de vibrato */
float CalPercnt(ffreq,fpercnt)
float fpercnt,ffreq;
{
float fdepth;
	fdepth=ffreq*fpercnt;
	return (fdepth);
}

float getnum()
{
	float x;
	char str[100];
	if (fscanf(fi, "%s", str) < 1)
	{
		fprintf(stderr, "missing number\n");
		exit(1);
	}
	if (*str >= 'a' && *str <= 'z')
	{
		if (!strcmp(str, "fff")) x = 1./8;
		else if (!strcmp(str, "ff")) x = 1./16;
		else if (!strcmp(str, "f")) x = 1./32;
		else if (!strcmp(str, "p")) x = 1./64;
		else if (!strcmp(str, "pp")) x = 1./128;
		else 
		{
			fprintf(stderr, "bad dynamic\n");
			exit(1);
		}
	}
	else if (sscanf(str, "%f", &x) < 1)
	{
		fprintf(stderr, "bad number\n");
		exit(1);
	}
	return (x);
}













